﻿using DataSharing.CityStateDataExchange;
using DataSharing.SendTask;
using DataSharing.SendTask.OtherPlatforms;
using DataSharing.YiBin.Enterprise;
using DataSharing.YiBin.ZzptPlatformsYB;
using Microsoft.Extensions.Logging;

namespace DataSharing.YiBin
{
    /// <summary>
    /// 执行推送任务
    /// </summary>
    public class YiBinSendTaskDataService : ISendTaskDataService
    {
        private readonly ILogger<YiBinSendTaskDataService> _logger;
        private readonly IEnterprisePusherProviderService _enterprisePusherProviderService;
        private readonly ICityStateDataExchangePusherProviderService _dataExchangePusherProviderService;
        private readonly IZzptPusherProviderService _zzptPusherProviderService;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="pusherProviderService"></param>
        /// <param name="enterprisePusherProviderService"></param>
        /// <param name="dataExchangePusherProviderService"></param>
        /// <param name="zzptPusherProviderService"></param>
        public YiBinSendTaskDataService(ILogger<YiBinSendTaskDataService> logger,
         IEnterprisePusherProviderService enterprisePusherProviderService,
         ICityStateDataExchangePusherProviderService dataExchangePusherProviderService,
         IZzptPusherProviderService zzptPusherProviderService)
        {
            _logger = logger;
            _enterprisePusherProviderService = enterprisePusherProviderService;
            _dataExchangePusherProviderService = dataExchangePusherProviderService;
            _zzptPusherProviderService = zzptPusherProviderService;
        }

        /// <summary>
        /// 执行推送任务
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task SendTask(DsWaitSendTaskOtherPlatforms dto, CancellationToken cancellationToken)
        {
            if (dto is null)
                return;
            _logger.LogWarning("这里是宜宾的服务--------------------------------------");
            switch (dto.PlatformSource)
            {
                case Share.Enums.EPlatformSource.Police110:
                    break;
                case Share.Enums.EPlatformSource.CityDataExchange:
                    await _dataExchangePusherProviderService.SendDataExchangeDataPusher(dto, cancellationToken);
                    break;
                case Share.Enums.EPlatformSource.ConvergenceMedia:
                    break;
                case Share.Enums.EPlatformSource.Enterprise:
                    await _enterprisePusherProviderService.SendEnterpriseDataPusher(dto, cancellationToken);
                    break;
                case Share.Enums.EPlatformSource.ZZPT:
                    await _zzptPusherProviderService.SendDataPusher(dto, cancellationToken);
                    break;
                default:
                    break;
            }
        }

    }
}
